/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes.utils;

import org.apache.ws.commons.schema.XmlSchema;
import org.apache.ws.commons.schema.XmlSchemaCollection;
import org.apache.ws.commons.schema.XmlSchemaExternal;
import org.apache.ws.commons.schema.XmlSchemaObjectCollection;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.jdbc.mediatypes.builtin.DefaultMediaTypeHandler;
import org.xml.sax.InputSource;

import javax.xml.transform.OutputKeys;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.util.*;

public class SchemaFileProcessor {

    private DefaultMediaTypeHandler defaultMediaTypeHandler;

    // remove this when it is not needed
    private int i;

    public SchemaFileProcessor(DefaultMediaTypeHandler defaultMediaTypeHandler) {
        this.defaultMediaTypeHandler = defaultMediaTypeHandler;
    }

    /**
     * saves the file to the reginstry after saving all its inclues and imports to the reginstry and
     * updating the schema locations accordingly.
     *
     * @param location - original schema location.
     * @throws FileNotFoundException
     */

    public String saveSchemaFileToRegistry(String location,
                                         String registryBasePath,
                                         boolean processInclueds,
                                         Resource metadata)
            throws RegistryException {

        return saveSchemaFileToRegistry(location, new HashMap(), registryBasePath, processInclueds,
                                 metadata);
    }

    /**
     * @param location           - original schema location
     * @param processedSchemaMap - this map contains schema source URI vs new schema locaitons.
     */
    public String saveSchemaFileToRegistry(String location,
                                         Map processedSchemaMap,
                                         String registryBasePath,
                                         boolean processInclueds,
                                         Resource metadata)
            throws RegistryException {

        XmlSchemaCollection xmlSchemaCollection = new XmlSchemaCollection();
        InputSource inputSource = new InputSource(location);
        // here we assue schema is correct. schema validation is beyond registry scope.
        // hence passes null to validator.
        XmlSchema xmlSchema = xmlSchemaCollection.read(inputSource, null);

        String baseUri = xmlSchema.getSourceURI();
        String xsdFileName = baseUri.substring(baseUri.lastIndexOf("/") + 1);
        String savedName = xsdFileName.substring(0, xsdFileName.indexOf(".")) + ".xsd";

        // this is not an inline wsdl schema. so pass null to change map.
        calculateNewSchemaNames(xmlSchema, processedSchemaMap, new HashSet(), false,
                                processInclueds);
        saveSchemaFileToRegistry(xmlSchema, processedSchemaMap, null, new HashSet(), false,
                                 registryBasePath, processInclueds, metadata);

        return savedName;
    }

    /**
     * calculate the new schema file names to save the schema. Here we can not save the schema file
     * as it is since there can be recursive imports. So what we have to do is to first determine
     * the schema names to be saved and then change the schema locations accordingly. In this method
     * first we iterate through the imports and includes and find the names. have used the
     * visitedSchemas variable to keep track of the visited schemas to avoid the recursion.
     *
     * @param xmlSchema          - schema to change save into the registry
     * @param processedSchemaMap - map to keep the source uri vs new schema location
     * @param visitedShemas      - already visited schemas source uris
     * @param isWsdlInlineSchema - this variable is used to check whether the given schema is an
     *                           inline schema of a wsdl. in that case we do not need to calculate a
     *                           name for that
     */
    public void calculateNewSchemaNames(XmlSchema xmlSchema,
                                        Map processedSchemaMap,
                                        Set visitedShemas,
                                        boolean isWsdlInlineSchema,
                                        boolean processInclues) {

        if (processInclues) {
            // first process the imports and includes
            XmlSchemaObjectCollection includes = xmlSchema.getIncludes();
            // set this as an visited schema to stop recursion
            visitedShemas.add(xmlSchema.getSourceURI());
            if (includes != null) {
                Object externalComponet ;
                XmlSchemaExternal xmlSchemaExternal ;
                XmlSchema innerSchema ;
                for (Iterator iter = includes.getIterator(); iter.hasNext();) {
                    externalComponet = iter.next();
                    if (externalComponet instanceof XmlSchemaExternal) {
                        xmlSchemaExternal = (XmlSchemaExternal)externalComponet;
                        innerSchema = xmlSchemaExternal.getSchema();
                        // we do not need to process it if it already process
                        // when calling with a different import.
                        if (!processedSchemaMap.containsKey(innerSchema.getSourceURI()) &&
                            !visitedShemas.contains(innerSchema.getSourceURI())) {
                            calculateNewSchemaNames(xmlSchemaExternal.getSchema(),
                                                    processedSchemaMap, visitedShemas, false,
                                                    processInclues);
                        }
                    }
                }
            }
        }

        // after processing includes and imports save the xml schema
        if (!isWsdlInlineSchema) {
            String baseUri = xmlSchema.getSourceURI();
            String xsdFileName = baseUri.substring(baseUri.lastIndexOf("/") + 1);
            String fileNameToSave = xsdFileName.substring(0, xsdFileName.indexOf(".")) + ".xsd";
            while (processedSchemaMap.containsValue(fileNameToSave)) {
                fileNameToSave = xsdFileName.substring(0, xsdFileName.indexOf(".")) + ++i + ".xsd";
            }
            // add this entry to the proccessed wsdl map
            processedSchemaMap.put(baseUri, fileNameToSave);
        }
    }

    /**
     * save the schemas to the registry. used the calcualted names in the processedSchemaMap to
     * change the schema locations.
     *
     * @param xmlSchema          - xml scheam to save
     * @param processedSchemaMap - already determined schema names
     * @param changeSchemaNames  - this contains the map between original schema names and the
     *                           change uris. this is used to update the wsdl inline schema imports
     *                           and inclueds when saving the wsdl file.
     * @param visitedShemas      - keep track of visited schemas to avoid recursion
     * @param isWsdlInlineSchema - if called from a inline wsdl schema.
     */
    public void saveSchemaFileToRegistry(XmlSchema xmlSchema,
                                         Map processedSchemaMap,
                                         Map changeSchemaNames,
                                         Set visitedShemas,
                                         boolean isWsdlInlineSchema,
                                         String registryBasePath,
                                         boolean processIncludes,
                                         Resource metadata) throws RegistryException {

        if (processIncludes) {
            // first process the imports and includes
            XmlSchemaObjectCollection includes = xmlSchema.getIncludes();
            // set this as an visited schema to stop recursion
            visitedShemas.add(xmlSchema.getSourceURI());
            if (includes != null) {
                Object externalComponet ;
                XmlSchemaExternal xmlSchemaExternal ;
                XmlSchema innerSchema ;
                for (Iterator iter = includes.getIterator(); iter.hasNext();) {
                    externalComponet = iter.next();
                    if (externalComponet instanceof XmlSchemaExternal) {
                        xmlSchemaExternal = (XmlSchemaExternal)externalComponet;
                        innerSchema = xmlSchemaExternal.getSchema();
                        if (!visitedShemas.contains(innerSchema.getSourceURI())) {
                            saveSchemaFileToRegistry(xmlSchemaExternal.getSchema(),
                                                     processedSchemaMap, null, visitedShemas, false,
                                                     registryBasePath, processIncludes, null);
                        }

                        // add the new name to changeschema map
                        // have to do before change the schema location
                        if (isWsdlInlineSchema) {
                            changeSchemaNames.put(xmlSchemaExternal.getSchemaLocation(),
                                                  processedSchemaMap.get(
                                                          innerSchema.getSourceURI()));
                        }
                        // set the new location
                        xmlSchemaExternal.setSchemaLocation((String)processedSchemaMap.get(
                                innerSchema.getSourceURI()));

                    }
                }
            }
        }

        // after processing includes and imports save the xml schema
        if (!isWsdlInlineSchema) {
            String fileNameToSave = (String)processedSchemaMap.get(xmlSchema.getSourceURI());
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            xmlSchema.write(byteArrayOutputStream);
            byte[] xsdContent = byteArrayOutputStream.toByteArray();

            Resource xsdResource = new Resource();
            if (metadata != null) {
                xsdResource.setMediaType(metadata.getMediaType());
                xsdResource.setDescription(metadata.getDescription());
            }
            xsdResource.setContent(xsdContent);

            String xsdPath;
            if (RegistryConstants.ROOT_PATH.equals(registryBasePath)) {
                xsdPath = RegistryConstants.ROOT_PATH + fileNameToSave;
            } else {
                xsdPath = registryBasePath + RegistryConstants.PATH_SEPARATOR + fileNameToSave;
            }

            defaultMediaTypeHandler.put(xsdPath, xsdResource);

            //xmlSchema.write(new FileOutputStream("repository/" + fileNameToSave),
            //                getDefaultOptionMap());
            // add this entry to the proccessed wsdl map
        }
    }

}
