/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.urlhandlers;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.*;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.utils.AuthorizationUtil;
import org.wso2.registry.jdbc.dao.CommentsDAO;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;

public class CommentURLHandler extends URLHandler {

    private static final Log log = LogFactory.getLog(CommentURLHandler.class);

    private CommentsDAO commentsDAO = new CommentsDAO();

    public CommentURLHandler(DataSource dataSource, Realm realm) {
        super(dataSource, realm);
    }

    /**
     * Accepts URL in the form <path>;comments:<comment-id>
     *
     * @param url
     * @return
     */
    public Resource get(String url) throws RegistryException {

        String[] parts = url.split(";");

        if (parts.length != 2) {
            return null;
        }

        if (parts[0].length() == 0) {
            return null;
        }

        if (!parts[1].startsWith("comments:")) {
            return null;
        }

        String commentID = parts[1].substring("comments:".length());
        if (!(commentID.length() > 0)) {
            return null;
        }

        long cID;
        try {
            cID = Long.parseLong(commentID);

        } catch (NumberFormatException e) {
            // note that this might not be an exceptional scenario. there could be a different URL
            // form, which contains strings after "comment".
            // it is just that it is not the URL we expect here
            return null;
        }

        Connection conn;
        try {
            conn = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }

        Comment comment = null;
        String resourcePath = "";
        try {
            comment = commentsDAO.getComment(cID, conn);
            resourcePath = commentsDAO.getResourcePathOfComment(cID, conn);

        } catch (SQLException e) {
            String msg = "Could not get the comment with ID: " + cID +
                    ". Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);

        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException ignore) {
                    log.info("exception.closing.db");
                }
            }
        }

        if (comment == null) {
            String msg = "Requested comment with ID: " + cID + " is not available.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        Resource resource = new Resource();
        resource.setPath(url);
        resource.setMediaType(RegistryConstants.COMMENT_MEDIA_TYPE);
        resource.setContent(comment.getText());
        resource.setAuthorUserName(comment.getUser());
        resource.setCreatedTime(comment.getTime());
        resource.setProperty("resourcePath", resourcePath);

        return resource;
    }

    public boolean delete(String path) throws RegistryException {

        String[] parts = path.split(";");

        if (parts.length != 2) {
            return false;
        }

        if (parts[0].length() == 0) {
            return false;
        }

        if (!parts[1].startsWith("comments:")) {
            return false;
        }

        String commentID = parts[1].substring("comments:".length());
        if (!(commentID.length() > 0)) {
            return false;
        }

        long cID;
        try {
            cID = Long.parseLong(commentID);

        } catch (NumberFormatException e) {
            // note that this might not be an exceptional scenario. there could be a different URL
            // form, which contains strings after "comment".
            // it is just that it is not the URL we expect here
            return false;
        }

        String userID = User.getCurrentUser();
        String authPath = AuthorizationUtil.getAuthorizationPath(path);
        String commentAuthor;

        Connection conn1 = null;
        try {
            conn1 = dataSource.getConnection();

            Comment comment = commentsDAO.getComment(cID, conn1);
            commentAuthor = comment.getUser();

        } catch (SQLException e) {

            String msg = "Failed to get the author of the comment.";
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            if (conn1 != null) {
                try {
                    conn1.close();
                } catch (SQLException e) {
                    log.info("exception.closing.db");
                }
            }
        }

        // check if the current user has permission to delete this comment.
        // users who have PUT permission on the commented resource can delete any comment on
        // that resource. Any user can delete his own comment.

        try {
            if (!userID.equals(commentAuthor) &&
                !realm.getAuthorizer().isUserAuthorized(userID, authPath, ActionConstants.PUT)) {

                String msg = "User: " + userID +
                        " is not authorized to delete the comment on the resource: " + authPath;
                log.info(msg);
                throw new AuthorizationFailedException(msg);
            }

        } catch (UserManagerException e) {
            //
        }


        Connection conn2;
        try {
            conn2 = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }

        try {

            commentsDAO.deleteComment(cID, conn2);

        } catch (SQLException e) {
            String msg = "Could not delete the comment with the path: " + path;
            throw new RegistryException(msg, e);
        } finally {
            if (conn2 != null) {
                try {
                    conn2.close();
                } catch (SQLException ignore) {
                    log.info("exception.closing.db");
                }
            }
        }

        return true;
    }
}
