/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.servlet.utils;

import org.wso2.registry.Registry;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.registry.secure.SecureRegistry;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

public class Utils {

    /**
     * Extract the paramters from the query string. e.g. for the URL http://localhost:8080/registry/d1/myresource?action=tag,tag_value=support
     * project we have the query string "action=tag,tag_value=support project" it should be
     * extracted to map as key=action, value=tag key=tag_value, value=support project
     *
     * @param queryString query string of the URL
     * @return Map containg paramter name --> paramter value pairs
     */
    public static Map getParameters(String queryString) {
        Map paramMap = new HashMap();
        String[] params = queryString.split(",");
        for (int i = 0; i < params.length; i++) {
            String[] param = params[i].split("=");
            paramMap.put(param[0].trim(), param[1].trim());
        }
        return paramMap;
    }

    /**
     * Returns the registry associated with the current session. If a user registry is not found,
     * new SecureRegistry instance is created with annoymous user and associated for the current
     * session.
     *
     * @param request Servlet request
     * @return SecureRegistry instance for the current session.
     * @throws org.wso2.registry.RegistryException
     *          : if something went wrong
     */
    public static synchronized SecureRegistry getSecureRegistry(HttpServletRequest request)
            throws RegistryException {
        SecureRegistry registry;
        Object o = request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);
        if (o != null) {
            registry = (SecureRegistry)o;
        } else {
            Registry coreRegistry = (Registry)request.
                    getSession().getServletContext().getAttribute(RegistryConstants.REGISTRY);
            RegistryRealm realm = (RegistryRealm)request.getSession().getServletContext().
                    getAttribute(RegistryConstants.REGISTRY_REALM);
            registry = new SecureRegistry(
                    RegistryConstants.ANONYMOUS_USER, "guest", coreRegistry, realm);

            request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, registry);
        }
        return registry;
    }

    public static synchronized void setSecureRegistry(
            HttpServletRequest request, SecureRegistry secureRegistry) {
        request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, secureRegistry);
    }

    public static boolean isLoggedIn(HttpServletRequest request) {

        SecureRegistry userRegistry =
                (SecureRegistry)request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);

        return userRegistry != null && !userRegistry.getUserID().equals(RegistryConstants.ANONYMOUS_USER);
    }

    public static void logInUser(HttpServletRequest request, String userName, String password)
            throws RegistryException {

        ServletContext context = request.getSession().getServletContext();
        Registry jdbcRegistry = (Registry)context.getAttribute(RegistryConstants.REGISTRY);

        RegistryRealm realm = (RegistryRealm)request.getSession().getServletContext().
                getAttribute(RegistryConstants.REGISTRY_REALM);

        SecureRegistry secureRegistry = new SecureRegistry(
                userName, password, jdbcRegistry, realm);

        request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, secureRegistry);
    }
}
